'''
Features for interacting with Plexon's Omniplex neural recording system
'''
import numpy as np
import os
from .hdf_features import SaveHDF
import glob
import datetime

sec_per_min = 60

class RelayPlexon(object):
    '''
    Sends the full data from eyetracking and motiontracking systems directly into Plexon
    '''
    def init(self):
        '''
        Secondary init function. See riglib.experiment.Experiment.init()
        Prior to starting the task, this 'init' sets up the NIDAQ card as a sink
        '''
        from riglib import sink
        sink_manager = sink.SinkManager.get_instance()
        self.nidaq = sink_manager.start(self.ni_out)
        super(RelayPlexon, self).init()

        # Find all the plexon files modified in the last day
        file_pattern = "/storage/plexon/*.plx"
        file_names = glob.glob(file_pattern)
        start_time = datetime.datetime.today() - datetime.timedelta(days=1)
        file_names = [fname for fname in file_names if datetime.datetime.fromtimestamp(os.stat(fname).st_mtime) > start_time]

        self.possible_filenames = file_names
        self.possible_filesizes = np.array([os.stat(fname).st_size for fname in self.possible_filenames])

    @property
    def ni_out(self):
        '''
        Specify the output interface; can be overridden in child classes as long as
        this method returns a class which has the same instance methods (close, register, send, sendMsg, etc.)
        '''
        from riglib.dio import nidaq
        return nidaq.SendAll

    @property
    def plexfile(self):
        '''
        Calculates the plexon file that's most likely associated with the current task
        based on the time at which the task ended and the "last modified" time of the
        plexon files located at /storage/plexon/
        '''
        if hasattr(self, '_plexfile'):
            return self._plexfile
        else:
            ## Ideally, the correct 'plexfile' will be the only file whose filesize has changed since the 'init' function ran..
            filesizes = np.array([os.stat(fname).st_size for fname in self.possible_filenames])
            inds, = np.nonzero(filesizes - self.possible_filesizes)
            if len(inds) == 1:
                print("only one plx file changed since the start of the task.")
                self._plexfile = self.possible_filenames[inds[0]]
                return self._plexfile

            ## Otherwise, try to find a file whose last modified time is within 60 seconds of the task ending; this requires fairly accurate synchronization between the two machines
            if len(self.event_log) < 1:
                self._plexfile = None
                return self._plexfile

            start = self.event_log[-1][2]
            files = "/storage/plexon/*.plx"
            files = sorted(glob.glob(files), key=lambda f: abs(os.stat(f).st_mtime - start))

            if len(files) > 0:
                tdiff = os.stat(files[0]).st_mtime - start
                if abs(tdiff) < sec_per_min:
                    self._plexfile = files[0]
                    return self._plexfile

            ## If both methods fail, return None; cleanup should warn the user that they'll have to link the plexon file manually
            self._plexfile = None
            return self._plexfile

    def run(self):
        '''
        Code to execute immediately prior to the beginning of the task FSM executing, or after the FSM has finished running.
        See riglib.experiment.Experiment.run(). This 'run' method stops the NIDAQ sink after the FSM has stopped running.
        '''
        try:
            super(RelayPlexon, self).run()
        finally:
            # Stop the NIDAQ sink
            self.nidaq.stop()

            # Remotely stop the recording on the plexon box
            import comedi
            import time
            com = comedi.comedi_open("/dev/comedi0")
            time.sleep(0.5)
            comedi.comedi_dio_bitfield2(com, 0, 16, 16, 16)

    def set_state(self, condition, **kwargs):
        '''
        Extension of riglib.experiment.Experiment.set_state. Send the name of the next state to
        plexon system and then proceed to the upstream set_state tasks.

        Parameters
        ----------
        condition : string
            Name of new state.
        **kwargs : dict
            Passed to 'super' set_state function

        Returns
        -------
        None
        '''
        self.nidaq.sendMsg(condition)
        super(RelayPlexon, self).set_state(condition, **kwargs)

    def cleanup(self, database, saveid, **kwargs):
        '''
        Function to run at 'cleanup' time, after the FSM has finished executing. See riglib.experiment.Experiment.cleanup
        This 'cleanup' method remotely stops the plexon file recording and then links the file created to the database ID for the current TaskEntry
        '''
        # Stop recording
        import comedi
        import time

        com = comedi.comedi_open("/dev/comedi0")
        comedi.comedi_dio_bitfield2(com, 0, 16, 16, 16)

        super(RelayPlexon, self).cleanup(database, saveid, **kwargs)

        # Sleep time so that the plx file has time to save cleanly
        time.sleep(2)
        dbname = kwargs['dbname'] if 'dbname' in kwargs else 'default'
        if self.plexfile is not None:
            if dbname == 'default':
                database.save_data(self.plexfile, "plexon", saveid, True, False)
            else:
                database.save_data(self.plexfile, "plexon", saveid, True, False, dbname=dbname)
        else:
            print('\n\nPlexon file not found properly! It will have to be manually linked!\n\n')

    @classmethod
    def pre_init(cls, saveid=None, **kwargs):
        '''
        Run prior to starting the task to remotely start recording from the plexon system
        '''
        if saveid is not None:
            import comedi
            import time

            com = comedi.comedi_open("/dev/comedi0")
            # stop any recording
            comedi.comedi_dio_bitfield2(com, 0, 16, 16, 16)
            time.sleep(0.1)
            # start recording
            comedi.comedi_dio_bitfield2(com, 0, 16, 0, 16)

            time.sleep(3)
            super(RelayPlexon, cls).pre_init(saveid=saveid, **kwargs)


class RelayPlexByte(RelayPlexon):
    '''
    Relays a single byte (0-255) to synchronize the rows of the HDF table(s) with the plexon recording clock.
    '''
    def init(self):
        '''
        Secondary init function. See riglib.experiment.Experiment.init()
        Prior to starting the task, this 'init' ensures that this feature is
        only used if the SaveHDF feature is also enabled.
        '''
        if not isinstance(self, SaveHDF):
            raise ValueError("RelayPlexByte feature only available with SaveHDF")
        super(RelayPlexByte, self).init()

    @property
    def ni_out(self):
        '''
        see documentation for RelayPlexon.ni_out
        '''
        from riglib.dio import nidaq
        return nidaq.SendRowByte


from .neural_sys_features import CorticalBMI
class PlexonBMI(CorticalBMI):
    @property
    def sys_module(self):
        from riglib import plexon
        return plexon




'''
Collections of shapes to render on screen
'''

import numpy as np
from OpenGL.GL import *
from OpenGL import GLUT as glut

from .xfm import Transform

class Model(object):
    def __init__(self, shader="default", color=(0.5, 0.5, 0.5, 1), 
        shininess=10, specular_color=(1.,1.,1.,1.)):
        '''
        Docstring Constructor for Model

        Parameters
        ----------
        shader: string
            OpenGL shading method?
        color: tuple of length 4
            (r, g, b, translucence)
        shininess: float
            ???
        specular_color: tuple of length 4
            color of the light source???
        '''
        self.shader = shader
        self.parent = None

        # This is different from self.xfm = Transform() for children of Model implemented with multiple inheritance
        super(Model, self).__setattr__("xfm", Transform())
        self.color = color
        self.shininess = shininess
        self.spec_color = specular_color

        # The orientation of the object, in the world frame
        self._xfm = self.xfm
        self.allocated = False
    
    def __setattr__(self, attr, xfm):
        '''Checks if the xfm was changed, and recaches the _xfm which is sent to the shader'''
        val = super(Model, self).__setattr__(attr, xfm)
        if attr == "xfm":
            self._recache_xfm()

        return val
    
    def _recache_xfm(self):
        '''
        For models with a parent, the transform of the current model must be cascaded with the parent model's transform.
        NOTE: this only goes one level up the graphics tree, so the transform is 
        always with respect to the parent's frame, not with respect to the world frame!
        '''
        if self.parent is not None:
            self._xfm = self.parent._xfm * self.xfm
        else:
            self._xfm = self.xfm
    
    def init(self):
        allocated = self.allocated
        self.allocated = True
        return allocated

    def rotate_x(self, deg, reset=False):
        self.xfm.rotate_x(np.radians(deg), reset=reset)
        self._recache_xfm()
        return self

    def rotate_y(self, deg, reset=False):
        self.xfm.rotate_y(np.radians(deg), reset=reset)
        self._recache_xfm()
        return self

    def rotate_z(self, deg, reset=False):
        self.xfm.rotate_z(np.radians(deg), reset=reset)
        self._recache_xfm()
        return self

    def translate(self, x, y, z, reset=False):
        self.xfm.translate(x,y,z, reset=reset)
        self._recache_xfm()
        return self
    
    def render_queue(self, shader=None):
        '''Yields the shader, texture, and the partial drawfunc for queueing'''
        if shader is not None:
            yield shader, self.draw, None
        else:
            yield self.shader, self.draw, None

    def draw(self, ctx, **kwargs):
        '''
        Parameters
        ----------
        ctx: ??????
        kwargs: optional keyword arguments
            Can specify 'color', 'specular_color', or 'shininess' of the object to draw (overriding the model's attributes)

        Returns: None
        '''
        
        glUniformMatrix4fv(ctx.uniforms.xfm, 1, GL_TRUE, self._xfm.to_mat().astype(np.float32))
        glUniform4f(ctx.uniforms.basecolor, *kwargs.pop('color', self.color))
        glUniform4f(ctx.uniforms.spec_color, *kwargs.pop('specular_color', self.spec_color))
        glUniform1f(ctx.uniforms.shininess, kwargs.pop('shininess', self.shininess))

        # glUniform4f(ctx.uniforms.basecolor, *(self.color if "color" not in kwargs else kwargs['color']))
        # glUniform4f(ctx.uniforms.spec_color, *(self.spec_color if "specular_color" not in kwargs else kwargs['spec_color']))
        # glUniform1f(ctx.uniforms.shininess, self.shininess if "shininess" not in kwargs else kwargs['shininess'])        

    def attach(self):
        assert self.parent is not None
        while self not in self.parent.models:
            self.parent.models.append(self)

    def detach(self):
        assert self.parent is not None
        while self in self.parent.models:
            self.parent.models.remove(self)


class Group(Model):
    def __init__(self, models=()):
        super(Group, self).__init__()
        self.models = []
        for model in models:
            self.add(model)
    
    def add(self, model):
        self.models.append(model)
        model.parent = self
        model._recache_xfm()

    def remove(self, model):
        # remove the redundant models
        if model not in self.models:
            return
        self.models.remove(model)
        del model

    def init(self):
        for model in self.models:
            model.init()
    
    def render_queue(self, xfm=np.eye(4), **kwargs):
        for model in self.models:
            for out in model.render_queue(**kwargs):
                yield out
    
    def draw(self, ctx, **kwargs):
        for model in self.models:
            model.draw(ctx, **kwargs)
    
    def __getitem__(self, idx):
        return self.models[idx]
    
    def _recache_xfm(self):
        super(Group, self)._recache_xfm()
        for model in self.models:
            model._recache_xfm()


builtins = dict([ (n[9:].lower(), getattr(glut, n)) 
                    for n in dir(glut) 
                    if "glutSolid" in n])
class Builtins(Model):
    def __init__(self, model, shader="fixedfunc", *args):
        super(Builtins, self).__init__(xfm)
        assert model in builtins
        self.model = builtins['model']
        self.args = args
    
    def draw(self, ctx):
        glPushMatrix()
        glLoadMatrixf(np.dot(xfm, self.xfm).ravel())
        self.model(*self.args)
        glPopMatrix()

class TriMesh(Model):   
    '''Basic triangle mesh model. Houses the GL functions for making buffers and displaying triangles'''
    def __init__(self, verts, polys, normals=None, tcoords=None, **kwargs):
        super(TriMesh, self).__init__(**kwargs)
        if verts.shape[1] == 3:
            verts = np.hstack([verts, np.ones((len(verts),1))])
        if normals.shape[1] == 3:
            normals = np.hstack([normals, np.ones((len(normals),1))])

        self.verts = verts
        self.polys = polys
        self.tcoords = tcoords
        self.normals = normals
    
    def init(self):
        allocated = super(TriMesh, self).init()
        if not allocated:
            self.vbuf = glGenBuffers(1)
            self.ebuf = glGenBuffers(1)
            glBindBuffer(GL_ARRAY_BUFFER, self.vbuf)
            glBufferData(GL_ARRAY_BUFFER, 
                self.verts.astype(np.float32).ravel(), GL_STATIC_DRAW)
            glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self.ebuf)
            glBufferData(GL_ELEMENT_ARRAY_BUFFER, 
                self.polys.astype(np.uint16).ravel(), GL_STATIC_DRAW)

            if self.tcoords is not None:
                self.tbuf = glGenBuffers(1)
                glBindBuffer(GL_ARRAY_BUFFER, self.tbuf)
                glBufferData(GL_ARRAY_BUFFER, 
                    self.tcoords.astype(np.float32).ravel(), GL_STATIC_DRAW)
            
            if self.normals is not None:
                self.nbuf = glGenBuffers(1)
                glBindBuffer(GL_ARRAY_BUFFER, self.nbuf)
                glBufferData(GL_ARRAY_BUFFER,
                    self.normals.astype(np.float32).ravel(), GL_STATIC_DRAW)
        return allocated
    
    def draw(self, ctx):
        super(TriMesh, self).draw(ctx)
        glEnableVertexAttribArray(ctx.attributes['position'])
        glBindBuffer(GL_ARRAY_BUFFER, self.vbuf)
        glVertexAttribPointer( ctx.attributes['position'],
            4, GL_FLOAT, GL_FALSE, 4*4, GLvoidp(0))
        
        if self.tcoords is not None and ctx.attributes['texcoord'] != -1:
            glEnableVertexAttribArray(ctx.attributes['texcoord'])
            glBindBuffer(GL_ARRAY_BUFFER, self.tbuf)
            glVertexAttribPointer(
                ctx.attributes['texcoord'], 2, 
                GL_FLOAT, GL_FALSE, 4*2, GLvoidp(0))

        if self.normals is not None and ctx.attributes['normal'] != -1:
            glEnableVertexAttribArray(ctx.attributes['normal'])
            glBindBuffer(GL_ARRAY_BUFFER, self.nbuf)
            glVertexAttribPointer(
                ctx.attributes['normal'], 4, 
                GL_FLOAT, GL_FALSE, 4*4, GLvoidp(0))
        
        glBindBuffer(GL_ELEMENT_ARRAY_BUFFER, self.ebuf);
        glDrawElements(
            GL_TRIANGLES,           # mode
            len(self.polys)*3,      # count
            GL_UNSIGNED_SHORT,      # type
            GLvoidp(0)              # element array buffer offset
        )
        glDisableVertexAttribArray(ctx.attributes['position'])
        if self.tcoords is not None and ctx.attributes['texcoord'] != -1:
            glDisableVertexAttribArray(ctx.attributes['texcoord'])
        if self.normals is not None and ctx.attributes['normal'] != -1:
            glDisableVertexAttribArray(ctx.attributes['normal'])

class FlatMesh(TriMesh):
    '''Takes smoothed or no-normal meshes and gives them a flat shading'''
    def __init__(self, verts, polys, normals=None, **kwargs):
        checked = dict()
        normals = []
        nverts = []
        npolys = []

        for i, poly in enumerate(polys):
            v1 = verts[poly[1]] - verts[poly[0]]
            v2 = verts[poly[2]] - verts[poly[0]]
            nvec = tuple(np.cross(v1, v2))

            npoly = []
            for v in poly:
                vert = tuple(verts[v])
                if (vert, nvec) not in checked:
                    checked[(vert, nvec)] = len(nverts)
                    npoly.append(len(nverts))
                    nverts.append(vert)
                    normals.append(nvec)
                else:
                    npoly.append(checked[(vert, nvec)])

            npolys.append(npoly)
        

        super(FlatMesh, self).__init__(np.array(nverts), np.array(npolys), 
            normals=np.array(normals), **kwargs)

class PolyMesh(TriMesh):
    def __init__(self, verts, polys, **kwargs):
        tripoly = []
        for poly in polys:
            for p in zip(poly[1:-1], poly[2:]):
                tripoly.append((poly[1],)+p)
        super(PolyMesh, self).__init__(verts, tripoly, **kwargs)


def obj_load(filename):

    facesplit = lambda x: x.split('/')
    verts, polys, normals, tcoords = [], [], [], []
    objfile = open(filename)
    for line in objfile:
        el = line.split()
        if el[0] == "#":
            pass
        elif el[0] == "v":
            verts.append(list(map(float, el[1:])))
        elif el[0] == "vt":
            tcoords.append(list(map(float, el[1:])))
        elif el[0] == "vn":
            normals.append(list(map(float, el[1:])))
        elif el[0] == "f":
            for v in el[1:]:
                pass
            list(map(facesplit, el[1:]))



'''
Code for getting data from kinarm
'''

import time
import numpy as np
from .source import DataSourceSystem
from . import kinarmsocket

class Kinarmdata(DataSourceSystem):
    '''
    Client for data streamed from kinarm, compatible with riglib.source.DataSource
    '''
    update_freq = 1000.

    #  dtype is the numpy data type of items that will go 
    #  into the (multi-channel, in this case) datasource's ringbuffer
    dtype = np.dtype((np.float, (3, 50)))

    def __init__(self, addr=("192.168.0.8", 9090)):
        '''
        Constructor for Kinarmdata and connect to server

        Parameters
        ----------
        addr : tuple of length 2
            (client (self) IP address, client UDP port)

        '''
        self.conn = kinarmsocket.KinarmSocket(addr)
        self.conn.connect()

    def start(self):
        '''
        Start receiving data
        '''
        self.data = self.conn.get_data()

    def stop(self):
        '''
        Disconnect from kinarmdata socket
        '''
        self.conn.disconnect()

    def get(self):
        '''
        Get a new kinarm sample
        '''
        # while True:
            # try:
            #     d = self.data.next()
            # except:
            #     break

        return next(self.data)

def make(cls=DataSourceSystem, *args, **kwargs):
    '''
    Docstring
    This ridiculous function dynamically creates a class with a new init function

    Parameters
    ----------

    Returns
    -------
    '''
    def init(self, *args, **kwargs):
        super(self.__class__, self).__init__(*args, **kwargs)
    
    dtype = np.dtype((np.float, (3, 50)))    
    return type(cls.__name__, (cls,), dict(dtype=dtype, __init__=init))


"""
Written by Pavi - Aug 2020 for reward system integration
Code for reward system used in Amy Orsborn lab
Functions list:
--------------
Class Basic -> reward(reward_time_s), test, calibrate, drain(drain_time_s)
"""

# import functions
from .gpio import ArduinoGPIO
from multiprocessing import Process
from . import singleton
import time
import os

from config.rig_defaults import reward as reward_settings

log_path = os.path.join(os.path.dirname(__file__), '../log/reward.log')

class Basic(singleton.Singleton):

    __instance = None
    timeout = 10

    def __init__(self):
        super().__init__()
        com_port = reward_settings['address']  # specify the port, based on windows/Unix, can find it on IDE or terminal
        self.board = ArduinoGPIO(port=com_port)
        self.reward_pin = reward_settings['digital_pin'] # pin on the arduino which should be connected to the reward system
        self.off()
        print('Reward system ready.')

    def on(self):
        """Open the solenoid."""
        self.board.write(self.reward_pin, 1)  # send a high signal to open the solenoid

    def off(self):
        """Close the solenoid."""
        self.board.write(self.reward_pin, 0) # send a low signal to close the solenoid

    def calibrate(self):
        """
        reward.calibrate() checks if the flow rate of the reward system is consistent. Since our reward system is gravity based,
        the flowrate depends on the setup. Currently, the flow rate is 2.8 mL/s and it take ~ 72 seconds to drain 200 mL of fluid.
        #TODO: Check flowrate inside booth setup
        #TODO: Check flowrate for different fluid (Apple juice)
        """
        self.drain(72)  # it takes around 72 seconds to drain 200 ml of fluid - Flow rate: 2.8 mL/s
        print('Check the breaker for calibration. You should notice 200 ml of fluid')

    def drain(self, drain_time=200):  # call this function to drain the reward system
        """
        this function is called from the webserver in ajax.reward_drain
        """
        self.on()
        time.sleep(drain_time)
        self.off()

    def async_drain(self, drain_time=200):
        """
        Calls drain() function in a separate process
        """
        if not hasattr(self, 'board'):
            return False # ignore request if board is not initialized
        p = Process(target=self.drain, args=((drain_time,)))
        p.start()

def open():
    try:
        reward = Basic.get_instance()
        return reward
    except:
        print("Reward system not found/ not active")
        import traceback
        import os
        import builtins
        traceback.print_exc(file=builtins.open(log_path, 'a'))



# Generated by Django 2.2.1 on 2019-12-02 08:26

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0003_auto_20191113_2158'),
    ]

    operations = [
        migrations.CreateModel(
            name='KeyValueStore',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('key', models.TextField()),
                ('value', models.TextField()),
            ],
        ),
    ]



'''
Various generic generators to combine with tasks. These appear to be mostly deprecated at this point
'''

import random
import itertools

import numpy as np

from .experiment import TrialTypes, Sequence


def block_random(*args, **kwargs):
    '''
    A generic block randomizer. 

    Parameters
    ----------

    Returns
    -------
    seq: list
        Block-random sequence of items where the length of each block is the product of the length of each the parameters being varied
    '''
    n_blocks = kwargs.pop('nblocks')
    inds = [np.arange(len(arg)) for arg in args]
    from itertools import product
    items = []
    for x in product(*inds):
        item = [arg[i] for arg,i in zip(args, x)]
        items.append(item)

    n_items = len(items)
    seq = []
    for k in range(n_blocks):
        inds = np.arange(n_items)
        np.random.shuffle(inds)
        for i in inds:
            seq.append(items[i])

    return seq

def runseq(exp, seq=None, reps=1):
    '''
    Turns a sequence into a Python generator by iterating through the sequence and yielding each sequence element

    Parameters
    ----------
    exp: Class object for task
        Used only if the experiment has 'trial_types' instead of a target sequence list
    seq: iterable
        Target sequence yielded to the task during the 'wait' state
    reps: int, optional, default=1
        Number of times to repeat the sequence

    Returns
    -------
    Generator object corresponding to sequence
    '''
    if hasattr(exp, "trial_types"):
        assert max(seq)+1 == len(exp.trial_types)
        for _ in range(reps):
            for s in seq:
                yield exp.trial_types[s]
    else:
        for _ in range(reps):
            for s in seq:
                yield s

##################################################
##### Old functions, for use with TrialTypes #####
##################################################
def endless(exp, probs=None):
    '''
    Deprecated
    '''
    if probs is None:
        while True:
            yield random.choice(exp.trial_types)
    else:
        assert len(probs) == len(exp.trial_types)
        probs = np.insert(np.cumsum(_fix_missing(probs)), 0, 0)
        assert probs[-1] == 1, "Probabilities do not add up to 1!"
        while True:
            rand = random.random()
            p = np.nonzero(rand < probs)[0].min()
            yield exp.trial_types[p-1]

def sequence(length, probs=2):
    '''
    Deprecated
    '''
    try:
        opts = len(probs)
        probs = _fix_missing(probs)
    except TypeError:
        opts = probs
        probs = [1 / float(opts)] * opts
    return np.random.permutation([i for i, p in enumerate(probs) for _ in range(int(length*p))])

def _fix_missing(probs):
    '''
    Deprecated
    '''
    total, n = list(map(sum, list(zip(*((i, 1) for i in probs if i is not None)))))
    if n < len(probs):
        p = (1 - total) / (len(probs) - n)
        probs = [i or p for i in probs]
    return probs

class AdaptiveTrials(object):
    '''
    Deprecated
    '''
    def __init__(self, exp, blocklen=8):
        assert issubclass(exp, TrialTypes)
        self.blocklen = blocklen
        self.trial_types = exp.trial_types
        self.new_block()

    def new_block(self):
        perblock = self.blocklen / len(self.trial_types)
        block = [[t]*perblock for t in self.trial_types]
        self.block = list(itertools.chain(*block))
        random.shuffle(self.block)

    def __next__(self):
        if len(self.block) < 1:
            self.new_block()
        return self.block[0]
    
    def correct(self):
        self.block.pop(0)
    
    def incorrect(self):
        ondeck = self.block.pop(0)
        self.block.append(ondeck)
        



import os
import glob
import json
import datetime
from db.tracker import models

def run(simulate=True):
    moved = set()
    for datafile in models.DataFile.objects.all():
        print(datafile)
        suffix = dict(eyetracker="edf", hdf="hdf", plexon="plx")
        date = datafile.entry.date
        thatday = datetime.date(date.year, date.month, date.day)
        nextday = thatday + datetime.timedelta(days=1)
        query = models.TaskEntry.objects.filter(date__gte=thatday, date__lte=nextday)
        entrynums = dict([(e, t) for t, e in enumerate(query.order_by("date"))])
        newname = "{subj}{time}_{num:02}.{suff}".format(
            subj=datafile.entry.subject.name[:4].lower(),
            time="%04d%02d%02d"%(thatday.year, thatday.month, thatday.day),
            num=entrynums[datafile.entry]+1, suff=suffix[datafile.system.name],
            )
        newpath = os.path.join(datafile.system.path, newname)
        oldpath = os.path.join(datafile.system.path, datafile.path)
        
        if not os.path.exists(oldpath):
            print("\tRemoving datafile: %s"%oldpath)
            if not simulate:
                datafile.remove()
        elif oldpath in moved:
            print("OH GOD moving already missing file!!!")
        elif datafile.system.name == "plexon":
            last = json.loads(datafile.entry.report)[-1][2]
            files = glob.glob("/storage/plexon/*.plx")
            files = sorted(files, key=lambda f: abs(os.stat(f).st_mtime - last))
            if len(files) > 0:
                tdiff = os.stat(files[0]).st_mtime - last
                print("\tPlexon file %f tdiff"%tdiff)
                if abs(tdiff) < 60:
                    if oldpath != files[0]:
                        print("\tFound plexon file %s, but recorded %s!"%(files[0], oldpath))
                        print("\tassociating %s to %s"%(files[0], newpath))
                        if not simulate:
                            moved.add(files[0])
                            os.rename(files[0], os.path.join(datafile.system.path, newname))
                            datafile.path = newname
                            datafile.save()
                    else:
                        print("\tPLX file is ok, just renaming %s to %s"%(oldpath, newpath))
                        if not simulate:
                            moved.add(oldpath)
                            os.rename(oldpath, newpath)
                            datafile.path = newname
                            datafile.save()
                else:
                    if oldpath != files[0]:
                        print("\tPlexfile doesn't match best %s vs. %s"%(files[0], oldpath))
                    else:
                        print("\tBad plexon file found, removing %s..."%oldpath)
                        if not simulate:
                            datafile.remove()
        else:
            #print "\tRenaming %s to %s"%(datafile.path, newpath)
            #if not simulate:
            #    moved.add(datafile.path)
            #    os.rename(datafile.path, newpath)
            #    datafile.path = newname
            #    datafile.save()
            pass



from .pyeCubeStream import eCubeStream
import numpy as np
import time

def remove_headstage_sources(ec):
    '''
    Removes headstages sources so they can be added again to the same eCube instance later.
    '''
    sources = ec.listadded()
    if len(sources[0]) > 0:
        for hs in np.unique(sources[0][0]):
            ec.remove(('Headstages', int(hs)))


from .tracker.models import DataFile, System, TaskEntry
import os
import sys

# sample_blackrock_files = [r'/storage/blackrock/test.nev', r'/storage/blackrock/test.ns5']
sample_blackrock_files = [r'/storage/blackrock/sampledata001.nev', r'/storage/blackrock/sampledata001.ns5', r'/storage/blackrock/sample_ecog.ns3']
# sample_blackrock_files = [r'/storage/blackrock/sample_ecog.ns3']

entry_num = sys.argv[1]
entry = TaskEntry.objects.get(id=entry_num)
system = System.objects.get(name='blackrock')

# get hdf associated with this model
datafile_list = DataFile.objects.filter(entry=entry).filter(system=System.objects.get(name='hdf'))
assert(len(datafile_list) == 1)
hdf_file = datafile_list[0]
hdf_fname_wo_ext = os.path.splitext(os.path.split(hdf_file.path)[1])[0]

for file_ in sample_blackrock_files:
	ext = os.path.splitext(os.path.split(file_)[1])[1]  # either .nev or .nsx (e.g., .ns5)
	path = hdf_fname_wo_ext + ext
	fullpath = os.path.join(r'/storage/blackrock', path)

	print(path)
	print(fullpath)
	if os.path.isfile(fullpath):
		raise Exception('File %s already exists!' % fullpath)
	else:
		print("Executing system command: sudo cp %s %s" % (file_, fullpath))
		print('')
		os.system("sudo cp %s %s" % (file_, fullpath))


		print('Saving DataFile object to database with following params:')
		print('entry:', entry)
		print('system:', system)
		print('path:', path)
		print('')
		DataFile(entry=entry, system=system, path=path).save()


